#!/usr/bin/python
# pip install htmlmin
# pip install cssmin
# pip install jsmin
# if platformIO has a different python instance, use this  C:\Users\redma\.platformio\penv\Scripts\pip.exe install htmlmin

import os
import re
import binascii
import subprocess

input_dir = "html"              # Sub folder of webfiles
output_file = "src/webcontent.h"


# version and revision extract

version = "undefined"
revision = "rev_NO_GIT"

try:

    revision = (
        subprocess.check_output(["git", "rev-list", "--count", "HEAD"])
        .strip()
        .decode("utf-8")
    )

    defFile = open("src/definitions.h")
    for line in defFile:
        if line.startswith("#define MAJOR_VERSION"):
            version = line.replace("#define MAJOR_VERSION ", "")
            version = version.strip()
        if line.startswith("#define BOARD_NAME"):
            boardName = line.replace("#define BOARD_NAME ", "").replace("\"","").replace('  ', '').replace('\t', '').replace('\n\n', '\n').replace('\n', '')
            boardname = version.strip()
    defFile.close()
except:
    print ("No Git installed. Version number will be skipped in a filename")

defFile = open("src/definitions.h")
for line in defFile:
    if line.startswith("#define MAJOR_VERSION"):
        version = line.replace("#define MAJOR_VERSION ", "")
        version = version.strip()
defFile.close()

##############################################

f_output = open(output_file, "w")
#URL_minify_js   = 'https://javascript-minifier.com/raw' # Website to minify javascript
#URL_minify_html = 'https://html-minifier.com/raw'        # Website to minify html
#URL_minify_css  = 'https://cssminifier.com/raw'         # Website to minify css

f_output.write("// This file is autogenerated. DO NOT MODIFY!!!\n\n")

def replaceWildCards(string):

    string = string.replace("%BOARD_NAME%", boardName);
    string = string.replace("%VERSION%", "v" + version + " - " + revision );

    return string

def write_to_file(file, data, dir=""):
    filename, file_extension = os.path.splitext(file)       # Split filename and file extension
    file_extension = file_extension.replace(".","")         # Remove puncuation in file extension

    dir = dir.replace(input_dir,"")                         # Remove the first directory(input_dir)
    dir = dir.replace("\\","/")                             # Change to /
    if (dir == "/index."):
        dir="/index.html"
    f_output.write("// " + dir + "\n")                      # Print comment
    f_output.write("const char* const data_" + filename + "_" + file_extension + "_path PROGMEM = \""+str(dir)+"\";\n")    # print path
    f_output.write("const char data_"+filename+"_"+file_extension+"[] PROGMEM = "+data+"\n\n")            # print binary data

    # f_output.write("#define data_" + filename + "_len " + str(data.count('0x')) +"\n")

def aschii2Hex(text):
    output_str = ""
    x = 1
    strLen = len(text)
    for character in text:
        output_str += hex(ord(character))

        if (x != strLen):
            output_str += ","
        x += 1
    return output_str

def minify_js(input_file):
    data = "R\"=====(\n"
    sourceFile = open (input_file, "r")
    with open (input_file, "r") as sourceFile:
        data+=replaceWildCards(sourceFile.read().replace('  ', '').replace('\t', '').replace('\n\n', '\n'))
    sourceFile.close()
    data += "\n)=====\";"
    return data

def minify_html(input_file):    
    data = "R\"=====(\n"
    with open (input_file, "r") as sourceFile:
        data+=replaceWildCards(sourceFile.read().replace('  ', '').replace('\t', '').replace('\n\n', '\n'))
    sourceFile.close()
    
    data += "\n)=====\";"
    return data

def minify_css(input_file):
    data = "R\"=====(\n"
    with open (input_file, "r") as sourceFile:
        data+=replaceWildCards(sourceFile.read().replace('  ', '').replace('\t', '').replace('\n\n', '\n'))
    sourceFile.close()
    
    data += "\n)=====\";"
    return data

for root, dirs, files in os.walk(input_dir, topdown=False):
    for name in files:   # for files
        if name.endswith(".js"):
            print(os.path.join(root, name))
            minified = minify_js(os.path.join(root, name))          # minify javascript
            write_to_file(name, minified, os.path.join(root, name)) # write to file

        elif name.endswith(".html"):
            print(os.path.join(root, name))
            minified = minify_html(os.path.join(root, name))        # minify html
            write_to_file(name, minified, os.path.join(root, name)) # write to file

        elif name.endswith(".css"):
            print(os.path.join(root, name))
            minified = minify_css(os.path.join(root, name))         # minify css
            write_to_file(name, minified, os.path.join(root, name)) # write to file

f_output.close()